@@ -18,6 +18,7 @@
  */
 
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
@@ -503,7 +504,7 @@ mp_cast_to_double(const MPNumber *x)
 
 
 static void
-mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, bool trim_zeroes, bool force_sign, GString *string)
+mp_cast_to_string_real(const MPNumber *x, int default_base, int base, int accuracy, bool trim_zeroes, bool force_sign, GString *string)
 {
     static char digits[] = "0123456789ABCDEF";
     MPNumber number, integer_component, fractional_component, temp;
@@ -574,36 +575,37 @@ mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, bool trim_zero
             g_string_prepend(string, "+");
     }
 
-    switch(base)
-    {
-    case 2:
-        g_string_append(string, "₂");
-        break;
-    case 8:
-        g_string_append(string, "₈");
-        break;
-    default:
-    case 10:
-        break;
-    case 16:
-        g_string_append(string, "₁₆");
-        break;
+    /* Append base suffix if not in default base */
+    if (base != default_base) {
+        const char *digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉"};
+        int multiplier = 1;
+        int b = base;
+
+        while (base / multiplier != 0)
+            multiplier *= 10;
+        while (multiplier != 1) {
+            int d;
+            multiplier /= 10;
+            d = b / multiplier;
+            g_string_append(string, digits[d]);
+            b -= d * multiplier;
+        }
     }
 }
 
 
 void
-mp_cast_to_string(const MPNumber *x, int base, int accuracy, bool trim_zeroes, char *buffer, int buffer_length)
+mp_cast_to_string(const MPNumber *x, int default_base, int base, int accuracy, bool trim_zeroes, char *buffer, int buffer_length)
 {
     MPNumber x_real, x_im;
     GString *string;
-    
+
     string = g_string_sized_new(buffer_length);
-    
+
     mp_real_component(x, &x_real);
     mp_imaginary_component(x, &x_im);
-    
-    mp_cast_to_string_real(&x_real, base, accuracy, trim_zeroes, FALSE, string);
+
+    mp_cast_to_string_real(&x_real, default_base, base, accuracy, trim_zeroes, FALSE, string);
     if (mp_is_complex(x)) {
         GString *s;
         gboolean force_sign = TRUE;
@@ -614,7 +616,7 @@ mp_cast_to_string(const MPNumber *x, int base, int accuracy, bool trim_zeroes, c
         }
 
         s = g_string_sized_new(buffer_length);
-        mp_cast_to_string_real(&x_im, 10, accuracy, trim_zeroes, force_sign, s);
+        mp_cast_to_string_real(&x_im, default_base, 10, accuracy, trim_zeroes, force_sign, s);
         if (strcmp(s->str, "1") == 0) {
             g_string_append(string, "i");
         }
@@ -637,6 +639,70 @@ mp_cast_to_string(const MPNumber *x, int base, int accuracy, bool trim_zeroes, c
 }
 
 
+void
+mp_cast_to_exponential_string(const MPNumber *x, int default_base, int base_, int max_digits, bool trim_zeroes, bool eng_format, char *buffer, int buffer_length)
+{
+    char fixed[1024], *c;
+    MPNumber t, z, base, base3, base10, base10inv, mantissa;
+    int exponent = 0;
+    GString *string;
+    const char *super_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹"};
+
+    string = g_string_sized_new(buffer_length);
+
+    mp_abs(x, &z);
+    if (mp_is_negative(x))
+        g_string_append(string, "−");
+    mp_set_from_mp(&z, &mantissa);
+
+    mp_set_from_integer(base_, &base);
+    mp_xpowy_integer(&base, 3, &base3);
+    mp_xpowy_integer(&base, 10, &base10);
+    mp_set_from_integer(1, &t);
+    mp_divide(&t, &base10, &base10inv);
+
+    if (!mp_is_zero(&mantissa)) {
+        while (!eng_format && mp_is_greater_equal(&mantissa, &base10)) {
+            exponent += 10;
+            mp_multiply(&mantissa, &base10inv, &mantissa);
+        }
+
+        while ((!eng_format &&  mp_is_greater_equal(&mantissa, &base)) ||
+                (eng_format && (mp_is_greater_equal(&mantissa, &base3) || exponent % 3 != 0))) {
+            exponent += 1;
+            mp_divide(&mantissa, &base, &mantissa);
+        }
+
+        while (!eng_format && mp_is_less_than(&mantissa, &base10inv)) {
+            exponent -= 10;
+            mp_multiply(&mantissa, &base10, &mantissa);
+        }
+
+        mp_set_from_integer(1, &t);
+        while (mp_is_less_than(&mantissa, &t) || (eng_format && exponent % 3 != 0)) {
+            exponent -= 1;
+            mp_multiply(&mantissa, &base, &mantissa);
+        }
+    }
+
+    mp_cast_to_string(&mantissa, default_base, base_, max_digits, trim_zeroes, fixed, 1024);
+    g_string_append(string, fixed);
+    if (exponent != 0) {
+        g_string_append_printf(string, "×10"); // FIXME: Use the current base
+        if (exponent < 0) {
+            exponent = -exponent;
+            g_string_append(string, "⁻");
+        }
+        snprintf(fixed, 1024, "%d", exponent);
+        for (c = fixed; *c; c++)
+            g_string_append(string, super_digits[*c - '0']);
+    }
+
+    strncpy(buffer, string->str, buffer_length);
+    g_string_free(string, TRUE);
+}
+
+
 static int
 char_val(char **c, int base)
 {
@@ -717,8 +783,38 @@ ends_with(const char *start, const char *end, const char *word)
 }
 
 
+// FIXME: Doesn't handle errors well (e.g. trailing space)
+static bool
+set_from_sexagesimal(const char *str, int length, MPNumber *z)
+{
+    int degrees = 0, minutes = 0;
+    char seconds[length+1];
+    MPNumber t;
+    int n_matched;
+
+    seconds[0] = '\0';
+    n_matched = sscanf(str, "%d°%d'%s\"", &degrees, &minutes, seconds);
+
+    if (n_matched < 1)
+        return true;
+    mp_set_from_integer(degrees, z);
+    if (n_matched > 1) {
+        mp_set_from_integer(minutes, &t);
+        mp_divide_integer(&t, 60, &t);
+        mp_add(z, &t, z);
+    }
+    if (n_matched > 2) {
+        mp_set_from_string(seconds, 10, &t);
+        mp_divide_integer(&t, 3600, &t);
+        mp_add(z, &t, z);
+    }
+
+    return false;
+}
+
+
 bool
-mp_set_from_string(const char *str, MPNumber *z)
+mp_set_from_string(const char *str, int default_base, MPNumber *z)
 {
     int i, base, negate = 0, multiplier = 0, base_multiplier = 1;
     const char *c, *end;
@@ -728,6 +824,9 @@ mp_set_from_string(const char *str, MPNumber *z)
     const char *fractions[]     = {"½", "⅓", "⅔", "¼", "¾", "⅕", "⅖", "⅗", "⅘", "⅙", "⅚", "⅛", "⅜", "⅝", "⅞", NULL};
     int numerators[]            = { 1,   1,   2,   1,   3,   1,   2,   3,   4,   1,   5,   1,   3,   5,   7};
     int denominators[]          = { 2,   3,   3,   4,   4,   5,   5,   5,   5,   6,   6,   8,   8,   8,   8};
+  
+    if (strstr(str, "°"))
+        return set_from_sexagesimal(str, strlen(str), z);
 
     /* Find the base */
     end = str;
@@ -747,7 +846,7 @@ mp_set_from_string(const char *str, MPNumber *z)
             break;
     }
     if (base_multiplier == 1)
-        base = 10;
+        base = default_base;
 
     /* Check if this has a sign */
     c = str;
